package com.jiuqi.grid.output;

import com.jiuqi.grid.CurrencyCellProperty;
import com.jiuqi.grid.GridCell;
import com.jiuqi.grid.GridData;
import com.jiuqi.util.Html;
import com.jiuqi.util.StringUtils;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

public class HTMLGridGenerator {
   private GridData gridData;
   private int gridHeight;
   private int gridWidth;
   private boolean printCellScript;
   protected static final Object NULL_PROP_VALUE = new Object();
   private static final String CELL_STYLE_PREFIX = "jqgs";
   private static final String CELL_STYLE_TOP = "t";
   private static final String CELL_STYLE_LEFT = "l";
   public static final int BROWSER_IE = 1;
   public static final int BROWSER_FIREFOX = 2;
   public static final int BROWSER_OTHER = 10;
   protected int browserType;
   public static final String SCRIPT_ATTR_NAME = "script";

   public HTMLGridGenerator() {
      this((GridData)null);
   }

   public HTMLGridGenerator(GridData gridData) {
      this.printCellScript = true;
      this.browserType = 1;
      this.gridData = gridData;
      this.gridHeight = 0;
      this.gridWidth = 0;
   }

   public GridData getGridData() {
      return this.gridData;
   }

   public void setGridData(GridData gridData) {
      this.gridData = gridData;
      this.gridHeight = 0;
      this.gridWidth = 0;
   }

   public boolean getPrintCellScript() {
      return this.printCellScript;
   }

   public void setPrintCellScript(boolean value) {
      this.printCellScript = value;
   }

   public final String generate() {
      StringBuffer output = new StringBuffer();
      this.generate(output);
      return output.toString();
   }

   public final void generate(StringBuffer output) {
      if (this.startGenerate()) {
         if (this.gridData != null) {
            for(int row = 0; row < this.gridData.getRowCount(); ++row) {
               this.startRow(row);
               IPropertySet rowProps = new PropertySet();
               this.getRowTRProperties(row, rowProps);
               output.append("<tr");
               this.printAttributes(output, rowProps);
               output.append(">");

               for(int col = 0; col < this.gridData.getColCount(); ++col) {
                  this.genTD(col, row, output);
               }

               output.append("</tr>\n");
               this.stopRow(row);
            }

            this.stopGenerate();
         }
      }
   }

   public final String getTableWidth() {
      return Integer.toString(this.calcGridWidth());
   }

   public final String getTableHeight() {
      return Integer.toString(this.calcGridHeight());
   }

   public final String getCellStyles() {
      StringBuffer stylesBuffer = new StringBuffer();
      this.getCellStyles(stylesBuffer);
      return stylesBuffer.toString();
   }

   public final void getCellStyles(StringBuffer output) {
      if (this.gridData != null && StringUtils.isEmpty(this.gridData.getDataClass())) {
         this.beforeGetCellStyles(output);
         GridCell[] styleCells = this.gridData.getCellSytles();

         int i;
         for(i = 0; i < styleCells.length; ++i) {
            output.append(".");
            output.append("jqgs");
            output.append(i);
            output.append(" {\n");
            getSingleCellStyle(styleCells[i], output, this.browserType);
            output.append("}\n");
         }

         styleCells = this.gridData.getCellStyles(2, 1, this.gridData.getColCount() - 1, 1, true);

         for(i = 0; i < styleCells.length; ++i) {
            if (styleCells[i] != null) {
               output.append(".");
               output.append(this.getCellClassName(styleCells[i]));
               output.append(" {\n");
               getSingleCellStyle(styleCells[i], output, this.browserType);
               output.append("}\n");
            }
         }

         styleCells = this.gridData.getCellStyles(1, 2, 1, this.gridData.getRowCount() - 1, true);

         for(i = 0; i < styleCells.length; ++i) {
            if (styleCells[i] != null) {
               output.append(".");
               output.append(this.getCellClassName(styleCells[i]));
               output.append(" {\n");
               getSingleCellStyle(styleCells[i], output, this.browserType);
               output.append("}\n");
            }
         }

         GridCell a1 = this.gridData.getCell(1, 1);
         output.append(".");
         output.append(this.getCellClassName(a1));
         output.append(" {\n");
         getSingleCellStyle(a1, output, this.browserType);
         output.append("}\n");
         this.afterGetCellStyles(output);
      }
   }

   public int getBrowserType() {
      return this.browserType;
   }

   public void setBrowserType(int type) {
      this.browserType = type;
   }

   private static void getSingleCellStyle(GridCell cell, StringBuffer output, int browseType) {
      String color = getHTMLColor(cell.getFontColor());
      String strBEdgeColor = getHTMLColor(cell.getBEdgeColor());
      String strREdgeColor = getHTMLColor(cell.getREdgeColor());
      String strLEdgeColor = getHTMLColor(cell.getLEdgeColor());
      String strTEdgeColor = getHTMLColor(cell.getTEdgeColor());
      String strREdgeStyle = getEdgeStyleString(cell.getREdgeStyle());
      String strBEdgeStyle = getEdgeStyleString(cell.getBEdgeStyle());
      String strTEdgeStyle = getEdgeStyleString(cell.getTEdgeStyle());
      String strLEdgeStyle = getEdgeStyleString(cell.getLEdgeStyle());
      String strBackColor = getHTMLColor(cell.getBackColor());
      int fontSize = cell.getFontSize();
      int getREdgeWidth = getBorderWidth(cell.getREdgeStyle());
      int getBEdgeWidth = getBorderWidth(cell.getBEdgeStyle());
      int getLEdgeWidth = getLBorderWidth(cell);
      int getTEdgeWidth = getTBorderWidth(cell);
      int borderStyle = cell.getBackStyle();
      output.append("");
      if (!color.equals("#000000") && !color.equals("") && !"window".equalsIgnoreCase(color)) {
         output.append("color:" + color + ";\n");
      }

      if (cell.getSilverHead()) {
         strTEdgeColor = cell.getRowNum() == 1 ? "#000000" : "#FFFFFF";
         strLEdgeColor = cell.getColNum() == 1 ? "#000000" : "#FFFFFF";
         strREdgeColor = "#000000";
         strBEdgeColor = "#000000";
         getTEdgeWidth = cell.getRowNum() == 1 ? 1 : 0;
         getREdgeWidth = 1;
         getBEdgeWidth = 1;
         getLEdgeWidth = 1;
         strBackColor = "buttonface";
      } else {
         if (cell.getRowNum() != 1) {
            getTEdgeWidth = 0;
         }

         if (cell.getColNum() != 1) {
            getLEdgeWidth = 0;
         }
      }

      output.append("border-top-color:" + strTEdgeColor + ";\n");
      output.append("border-left-color:" + strLEdgeColor + ";\n");
      output.append("border-right-color:" + strREdgeColor + ";\n");
      output.append("border-bottom-color:" + strBEdgeColor + ";\n");
      output.append("border-top-style:" + strTEdgeStyle + ";\n");
      output.append("border-left-style:" + strLEdgeStyle + ";\n");
      output.append("border-right-style:" + strREdgeStyle + ";\n");
      output.append("border-bottom-style:" + strBEdgeStyle + ";\n");
      output.append("border-top-width:" + getTEdgeWidth + ";\n");
      output.append("border-left-width:" + getLEdgeWidth + ";\n");
      output.append("border-right-width:" + getREdgeWidth + ";\n");
      output.append("border-bottom-width:" + getBEdgeWidth + ";\n");
      output.append("font-size:" + fontSize + "pt;\n");
      if (cell.getFontBold()) {
         output.append("font-weight:bold;\n");
      }

      String halign = getCellHorzAlign(cell);
      String valign = getCellVertAlign(cell);
      if (cell.getVertText() && browseType == 1) {
         output.append("direction:inherit;\n");
         output.append("writing-mode:tb-rl;\n");
         if (!halign.equals("")) {
            output.append("vertical-align:" + halignToValign(halign) + ";\n");
         }

         output.append("text-align:" + valignToHalign(valign) + ";\n");
      } else {
         if (!halign.equals("") && !halign.equals("left")) {
            output.append("text-align:" + halign + ";\n");
         }

         output.append("vertical-align:" + valign + ";\n");
      }

      int indent = cell.getIndent();
      if (indent != 0) {
         output.append("text-indent:" + indent + "em;");
      }

      if (cell.getFontItalic()) {
         output.append("font-style:italic;\n");
      }

      String fontName;
      if (cell.getFontUnderLine() || cell.getFontStrikeOut()) {
         fontName = "";
         fontName = fontName + (cell.getFontUnderLine() ? "underline " : "");
         fontName = fontName + (cell.getFontStrikeOut() ? "line-through " : "");
         output.append("text-decoration:" + fontName + ";\n");
      }

      fontName = cell.getFontName();
      if (!"".equals(fontName.trim())) {
         fontName = StringUtils.replace(fontName, "\"", "");
         output.append("font-family:\"" + fontName + "\";\n");
      }

      if (borderStyle == 0) {
         output.append("background-color:#ffffff;\n");
      } else if (!strBackColor.equals("")) {
         output.append("background-color:" + strBackColor + ";\n");
      }

      if (cell.getWrapLine()) {
         output.append("word-wrap:break-word;\n");
         output.append("word-break:break-all;\n");
      }

      output.append("overflow:hidden;\n");
   }

   private static String halignToValign(String halign) {
      if ("left".equals(halign)) {
         return "bottom";
      } else if ("center".equals(halign)) {
         return "middle";
      } else {
         return "right".equals(halign) ? "top" : "";
      }
   }

   private static String valignToHalign(String valign) {
      if ("top".equals(valign)) {
         return "left";
      } else if ("middle".equals(valign)) {
         return "center";
      } else {
         return "bottom".equals(valign) ? "right" : "";
      }
   }

   private static String getHTMLColor(int colorValue) {
      return Html.getHtmlColor(colorValue);
   }

   private static String getCellHorzAlign(GridCell cell) {
      switch (cell.getHorzAlign()) {
         case 1:
            return "left";
         case 2:
            return "right";
         case 3:
            return "center";
         default:
            return "";
      }
   }

   private static String getCellVertAlign(GridCell cell) {
      switch (cell.getVertAlign()) {
         case 1:
            return "top";
         case 2:
            return "bottom";
         default:
            return "middle";
      }
   }

   private static String getEdgeStyleString(int styleValue) {
      switch (styleValue) {
         case 0:
            return "solid";
         case 1:
            return "none";
         case 2:
            return "solid";
         case 3:
            return "dashed";
         case 4:
         case 9:
         case 15:
            return "dotted";
         case 5:
         case 6:
         case 8:
         case 10:
         case 11:
         case 12:
         case 13:
         case 14:
         default:
            return styleValue >= 10 ? "double" : "";
         case 7:
            return "double";
      }
   }

   private static int getBorderWidth(int edgeStyle) {
      if (edgeStyle == 10) {
         return 3;
      } else if (edgeStyle > 13) {
         return 3;
      } else {
         return edgeStyle > 4 ? 2 : 1;
      }
   }

   private static int getLBorderWidth(GridCell cell) {
      if (cell.getColNum() == 1) {
         return getBorderWidth(cell.getLEdgeStyle());
      } else {
         return cell.getBackStyle() == 28 ? 1 : 0;
      }
   }

   private static int getTBorderWidth(GridCell cell) {
      if (cell.getRowNum() == 1) {
         return getBorderWidth(cell.getTEdgeStyle());
      } else {
         return cell.getBackStyle() == 28 ? 1 : 0;
      }
   }

   protected static String htmlEncoding(String inString) {
      return Html.encodeText(inString);
   }

   private void genTD(int col, int row, StringBuffer output) {
      GridCell cell = this.gridData.getCell(col, row);
      this.startCell(cell);
      IPropertySet cellProps = new PropertySet();
      this.getCellTDProperties(cell, cellProps);
      String innerHTML = this.genTDContent(cell);
      output.append("<td");
      this.printAttributes(output, cellProps);
      output.append(">");
      output.append(innerHTML);
      output.append("</td>");
      this.stopCell(cell);
   }

   private void printAttributes(StringBuffer output, IPropertySet props) {
      Iterator propItr = props.iterator();

      while(propItr.hasNext()) {
         IPropertySet.IEntry curProp = (IPropertySet.IEntry)propItr.next();
         String key = curProp.getKey();
         Object value = curProp.getValue();
         output.append(" ");
         output.append(key);
         if (value != NULL_PROP_VALUE && value != null) {
            output.append("='");
            if (value instanceof Boolean) {
               Boolean boolValue = (Boolean)value;
               output.append(boolValue ? "1" : "0");
            } else {
               output.append(value);
            }

            output.append("'");
         }
      }

   }

   protected String genTDContent(GridCell cell) {
      String innerHTML = null;
      switch (cell.getDataType()) {
         case 3:
            CurrencyCellProperty currency = new CurrencyCellProperty(cell);
            if (currency.getShowCurrencyBox()) {
               int showType = 0;
               if (currency.getBigChineseChar()) {
                  showType = 2;
               } else if (currency.getChineseNumber()) {
                  showType = 1;
               }

               String data = cell.getCellData();
               if (StringUtils.isEmpty(data)) {
                  data = "0";
               }

               CurrencyBoxHTMLGenerator boxGen = new CurrencyBoxHTMLGenerator(currency.getDecimal(), currency.getPrecision(), data, showType, currency.getUnitString());
               innerHTML = boxGen.generate();
            } else {
               innerHTML = htmlEncoding(currency.getShowText(cell.getCellData()));
            }
            break;
         case 4:
         case 5:
         case 7:
         case 8:
         default:
            if (cell.getColNum() == 0 && cell.getRowNum() == 0) {
               innerHTML = "";
            } else if (cell.getColNum() == 0) {
               innerHTML = Integer.toString(cell.getRowNum());
            } else if (cell.getRowNum() == 0) {
               innerHTML = getABCValue(cell.getColNum());
            } else {
               innerHTML = this.getCellTDContent(cell);
            }

            if (innerHTML != null && innerHTML.length() > 0) {
               innerHTML = htmlEncoding(innerHTML);
            }
            break;
         case 6:
            innerHTML = this.getCellImageHTML(cell);
            break;
         case 9:
            innerHTML = this.getCellHotLinkHTML(cell);
      }

      return innerHTML != null && innerHTML.length() != 0 ? innerHTML : "&nbsp;";
   }

   private static String getABCValue(int value) {
      if (value < 0) {
         return "";
      } else {
         String returnStr = "";
         int modValue = value % 26;
         char returnInt = (char)("A".charAt(0) + modValue);
         char[] a = new char[]{returnInt};
         returnStr = String.valueOf(a);
         int divValue = value / 26;
         if (divValue >= 1) {
            returnStr = getABCValue(divValue - 1) + returnStr;
         }

         return returnStr;
      }
   }

   private String getCellImageHTML(GridCell cell) {
      String imgRef = cell.getImageReference();
      if (imgRef != null && imgRef.length() != 0) {
         StringBuffer result = new StringBuffer();
         result.append("<img");
         if (cell.getHorzAlign() == 5 && !this.gridData.getColAutoSize(cell.getColNum())) {
            result.append(" width=\"");
            result.append(cell.getWidth());
            result.append("\"");
         }

         if (cell.getVertAlign() == 5 && !this.gridData.getRowAutoSize(cell.getRowNum())) {
            result.append(" height=\"");
            result.append(cell.getHeight());
            result.append("\"");
         }

         result.append(" src=\"");
         result.append(StringUtils.replace(this.getRealLinkAddr(imgRef), "\\", "/"));
         result.append("\"/>");
         return result.toString();
      } else {
         return null;
      }
   }

   private String getCellHotLinkHTML(GridCell cell) {
      String[] linkInfos = cell.getLinkInformation();
      if (linkInfos != null && linkInfos.length == 3) {
         if (linkInfos[1] != null && linkInfos[1].length() != 0) {
            StringBuffer result = new StringBuffer();
            String content = StringUtils.replace(linkInfos[1], "\"", "'");
            if (isJavaScriptLink(content)) {
               result.append("<a href=\"javascript:");
               result.append(content);
            } else {
               result.append("<a href=\"");
               result.append(this.getRealLinkAddr(content));
            }

            result.append("\"");
            if (linkInfos[2] != null && linkInfos[2].length() != 0) {
               result.append(" target=\"");
               result.append(linkInfos[2]);
               result.append("\"");
            }

            result.append(">");
            if (linkInfos[0] != null && linkInfos[0].length() != 0) {
               result.append(htmlEncoding(linkInfos[0]));
            } else {
               result.append("&nbsp;");
            }

            result.append("</a>");
            return result.toString();
         } else {
            return linkInfos[0] != null && linkInfos[0].length() != 0 ? htmlEncoding(linkInfos[0]) : null;
         }
      } else {
         return "";
      }
   }

   private static boolean isJavaScriptLink(String content) {
      if (content != null && content.length() != 0) {
         int pos = content.indexOf(40);
         if (pos > 0 && pos < content.length() - 1) {
            pos = content.indexOf(41);
            return pos == content.length() - 1;
         } else {
            return false;
         }
      } else {
         return false;
      }
   }

   public static String makeCellID(int col, int row) {
      return "cell_" + Integer.toString(row) + "_" + Integer.toString(col);
   }

   private String getRealLinkAddr(String link) {
      String context = this.getContextPath();
      if (StringUtils.isEmpty(context)) {
         return link;
      } else if (link.toLowerCase().startsWith("http:")) {
         return link;
      } else {
         return link.startsWith("/") ? context + link : context + "/" + link;
      }
   }

   private int calcGridWidth() {
      if (this.gridWidth == 0 && this.gridData != null) {
         for(int i = 0; i < this.gridData.getColCount(); ++i) {
            if (this.gridData.getColVisible(i)) {
               this.gridWidth += this.gridData.getColWidths(i);
            }
         }
      }

      return this.gridWidth;
   }

   private int calcGridHeight() {
      if (this.gridHeight == 0 && this.gridData != null) {
         for(int i = 0; i < this.gridData.getRowCount(); ++i) {
            if (this.gridData.getRowVisible(i)) {
               this.gridHeight += this.gridData.getRowHeights(i);
            }
         }
      }

      return this.gridHeight;
   }

   protected String getContextPath() {
      return null;
   }

   protected boolean startGenerate() {
      return true;
   }

   protected void stopGenerate() {
   }

   protected void startRow(int row) {
   }

   protected void stopRow(int row) {
   }

   protected void startCell(GridCell cell) {
   }

   protected void stopCell(GridCell cell) {
   }

   protected void getRowTRProperties(int row, IPropertySet rowProps) {
      String styleStr = "";
      if (!this.gridData.getRowVisible(row) || row == 0) {
         styleStr = "display:none;";
      }

      GridCell cell = this.gridData.getCell(0, row);
      if (!this.gridData.getRowAutoSize(row)) {
         styleStr = styleStr + "height:" + cell.getCellHeight() + "px;";
      }

      rowProps.put("style", styleStr);
   }

   protected void getCellTDProperties(GridCell cell, IPropertySet cellProps) {
      String className = this.getCellClassName(cell);
      if (StringUtils.isEmpty(className) && StringUtils.isEmpty(this.gridData.getDataClass())) {
         className = "jqgs" + this.gridData.getCellStyleIndex(cell.getColNum(), cell.getRowNum());
      }

      if (!StringUtils.isEmpty(className)) {
         cellProps.put("class", className);
      }

      int rowSpan;
      int visibleNumber;
      int i;
      if (cell.getColSpan() > 1) {
         rowSpan = cell.getColSpan();
         visibleNumber = cell.getColSpan();

         for(i = cell.getColNum() + 1; i < cell.getColNum() + rowSpan; ++i) {
            if (!this.gridData.getColVisible(i)) {
               --visibleNumber;
            }
         }

         cellProps.put("colSpan", new Integer(visibleNumber));
      }

      if (cell.getRowSpan() > 1) {
         rowSpan = cell.getRowSpan();
         visibleNumber = cell.getRowSpan();

         for(i = cell.getRowNum() + 1; i < cell.getRowNum() + rowSpan; ++i) {
            if (!this.gridData.getRowVisible(i)) {
               --visibleNumber;
            }
         }

         cellProps.put("rowSpan", new Integer(visibleNumber));
      }

      if (this.browserType != 2 && cell.getRowNum() == 0 && !this.gridData.getColAutoSize(cell.getColNum())) {
         cellProps.put("width", new Integer(cell.getCellWidth()));
      }

      if (!cell.getWrapLine()) {
         cellProps.put("nowrap", NULL_PROP_VALUE);
      }

      if (cell.getFitFontSize()) {
         cellProps.put("fft", Integer.toString(1));
      }

      String styleStr;
      if (this.printCellScript) {
         styleStr = cell.getScript();
         if (!StringUtils.isEmpty(styleStr)) {
            cellProps.put("script", styleStr);
         }
      }

      styleStr = "";
      if (cell.getColNum() == 0) {
         styleStr = styleStr + "display:none;";
      }

      if ((this.browserType == 2 || cell.getCellWidth() == 0) && !this.gridData.getColAutoSize(cell.getColNum())) {
         styleStr = styleStr + "width:" + cell.getCellWidth() + "px;";
      }

      if (cell.getCellLeft() != cell.getColNum() || cell.getCellTop() != cell.getRowNum()) {
         styleStr = styleStr + "display:none;";
         cellProps.put("oCI", String.valueOf(cell.getColNum() - cell.getCellLeft()));
      }

      if (!this.gridData.getColVisible(cell.getColNum())) {
         styleStr = styleStr + "display:none;";
      }

      if (cell.getVertText() && this.browserType == 1) {
         styleStr = styleStr + "width:" + cell.getWidth() + ";";
      }

      if (!styleStr.equals("")) {
         cellProps.put("style", styleStr);
      }

      if (cell.getCellData() != null && cell.getDataType() != 9 && cell.getDataType() != 6) {
         cellProps.put("rV", cell.getCellData());
      }

   }

   protected String getCellClassName(GridCell cell) {
      String className = cell.getCssClass();
      if (StringUtils.isEmpty(className) && StringUtils.isEmpty(this.gridData.getDataClass())) {
         if (cell.getColNum() == 1 && cell.getRowNum() == 1) {
            className = "tA1";
         } else {
            className = "jqgs" + Integer.toString(this.gridData.getCellStyleIndex(cell.getColNum(), cell.getRowNum()));
            if (cell.getColNum() == 1) {
               className = className + "l";
            } else if (cell.getRowNum() == 1) {
               className = className + "t";
            }
         }
      }

      return className;
   }

   protected String getCellTDContent(GridCell cell) {
      return cell.getShowText();
   }

   protected void beforeGetCellStyles(StringBuffer output) {
   }

   protected void afterGetCellStyles(StringBuffer output) {
   }

   protected static final class PropertySet implements IPropertySet {
      private Map finder = new HashMap();
      private List entries = new ArrayList();

      public void put(String key, Object value) {
         Entry entry = (Entry)this.finder.get(key);
         if (entry == null) {
            entry = new Entry(key, value);
            this.finder.put(key, entry);
            this.entries.add(entry);
         } else {
            entry.setValue(value);
         }

      }

      public void remove(String key) {
         Entry entry = (Entry)this.finder.remove(key);
         if (entry != null) {
            this.entries.remove(entry);
         }

      }

      public Iterator iterator() {
         return this.entries.iterator();
      }

      public void clear() {
         this.finder.clear();
         this.entries.clear();
      }

      public static final class Entry implements IPropertySet.IEntry {
         private String key;
         private Object value;

         public Entry(String key, Object value) {
            this.key = key;
            this.value = value;
         }

         public String getKey() {
            return this.key;
         }

         public Object getValue() {
            return this.value;
         }

         public void setValue(Object value) {
            this.value = value;
         }
      }
   }
}
